{---------------------------------------------------------------------------}
{ BitMachine Program: Reaction                                              }
{ DATE:         (See Version, below)                                        }
{ PROGRAMMER:   Terry King                                                  }
{ WHAT IT DOES: Measures Reaction time for a User. Needs Radio Shack type   }
{    276-143c   High Output Infared Light Emitting Diode (for beam)         }
{    276-145a   Infared Phototransistor                  (for sensor)       }
{                                                                           }
{ CHANGES:  Uses  'new' Bit layout for sensors/LED's: See Below             }
{---------------------------------------------------------------------------}
PROGRAM  Reaction;

{---------------------------------------------------------------------------}
{               U S E S : WHAT OTHER PASCAL "UNITS" YOU WILL USE            }
{                                                                           }
{ NOTE: Other sets of tools or procedures that other epople have created    }
{       that you want to re-use here.                                       }
{---------------------------------------------------------------------------}

USES
  CRT,
  TP_PAR,                       { Parallel Port Support }
  HrTimers,                     { High Resolution Timers }
  PcTimers,                     { PC Timer Trigger functions}
  Hexbin,                       { Hex, Binary output }
  Mwindow;                      { DOS Window support }

{---------------------------------------------------------------------------}
{               C O N S T A N T S: FOR YOUR DESIGN                          }
{                                                                           }
{ NOTE: Here, you have to tell what is hooked up to the Digital In and      }
{       Digital Out Bits! Also other 'constants' for your design.           }
{---------------------------------------------------------------------------}
CONST

MAIN          = 1     ; {WindowID's}
ERROR         = 2     ;
STATUS        = 3     ;
INFO          = 4     ;

{-------( Digital IN Bits: What they are being used for:  )--- -------------}
In0            =  $01;      { Pin 10 on Parallel Port }
In1            =  $02;      { Pin 11 on Parallel Port }
TopSensor      =  $04;      { Pin 12 on Parallel Port }
BotSensor      =  $08;      { Pin 13 on Parallel Port }
In4            =  $10;      { Pin 14 on Parallel Port }
In5            =  $20;      { Pin 15 on Parallel Port }
In6            =  $40;      { Pin 16 on Parallel Port }
In7            =  $80;      { Pin 17 on Parallel Port }

SensorPolarity =  $00;      {Opto sensors are normally '1', go to '0' in light}


{-------( Digital OUT Bits: What they are being used for:  )------  }

Out0        = 0;                                  { Pin 2  on Parallel Port }
Out1        = 1;                                  { Pin 3  on Parallel Port }
Out2        = 2;                                  { Pin 4  on Parallel Port }
Out3        = 3;   {Unused here}                  { Pin 5  on Parallel Port }
BlinkLED    = 4;   {Also Strobe1 in STROBE}       { Pin 6  on Parallel Port }
ToneGen     = 5;   {Also Strobe2 in STROBE}       { Pin 7  on Parallel Port }
BotBeamLED  = 6;                                  { Pin 8  on Parallel Port }
TopBeamLED  = 7;                                  { Pin 9  on Parallel Port }

{---------------------------------------------------------------------------}
{               T Y P E S        : FOR YOUR DESIGN                          }
{                                                                           }
{ NOTE: Here, you define data structures called TYPEs for your design       }
{---------------------------------------------------------------------------}
{TYPE}

{---------------------------------------------------------------------------}
{               V A R I A B L E S: FOR YOUR MACHINE DESIGN                  }
{                                                                           }
{ NOTE: Here, you have to tell what variables you will use to hold          }
{       temporary values for things. You can add lots more of these here.   }
{---------------------------------------------------------------------------}
VAR {MainLine}
    tbuff       : string;     { Strings are groups of characters        }
    Version     : string;
    Option      : string;
    Key         : Char;
    n           : integer;    { Integers are whole numbers up to 32,767 }
    TopTrigger  : TRIGGER_REC_TYPE;  {Top Sensor}
    BotTrigger  : TRIGGER_REC_TYPE;  {bottom Sensor}
    Time1,Time2 : HRTime_Rec_type;
    Done        : Boolean;
    WindowMode  : boolean;    {Windowed Display Enabled }
    UserSound,
    UserLight   : Boolean;    { Wether user gets beep, blink or both }
    Fault       : Boolean;    {User Faulted}
    DelayTimeMs : Word;       { for Random delay }

{---------------------------------------------------------------------------}
{  PROCEDURES AND FUNCTIONS      : FOR YOUR DESIGN                          }
{                                                                           }
{ NOTE: Here, you write procedures and functions that will be used          }
{       later in the Main Program.                                          }
{---------------------------------------------------------------------------}

PROCEDURE UpdateInfo;

  BEGIN
    SelectWindow(INFO);
    GoToXY(1,1);
    IF UserLight
      THEN write('SIGNAL: LIGHT')
      ELSE write('        ONLY ');
    GoToXY(1,2);
    IF UserSound
      THEN write('SIGNAL: SOUND')
      ELSE write('        ONLY ');
    SelectWindow(MAIN);
  END;


(*Ŀ
   MAIN PROGRAM:                                                           
  *)
BEGIN

 Version    := '1.03  09/11/96  USES TOP BEAM!';

 WindowMode := TRUE;
 UserSound  := TRUE;
 UserLight  := TRUE;
 Fault      := FALSE;

 WriteDO($00);

{-----------------( Set up Windows if enabled )-----------------}
 IF WindowMode THEN
   BEGIN
     ClrScr;
     WindowFrame(1,  1,  80, 15,
       (' BITS&BYTES:  REACTION TIME: ' + Version  ),
                 Yellow , Black, MAIN);
     WindowFrame(1,  15, 59, 25, 'MESSAGES', White  , Green, STATUS);
     WindowFrame(60, 15, 80, 25, 'INFO'    , White  , Blue , INFO);
     SelectWindow(MAIN);
   END; {Set Up Windows}


  TriggerSetup(TopTrigger, TopSensor, SensorPolarity, 1);
  TriggerSetup(BotTrigger, BotSensor, SensorPolarity, 1);

  Writeln('REACTION TIME PROGRAM');
  writeln;
  Writeln('YOU WILL HAVE BOTH L (LIGHT) and S (SOUND) SIGNAL.');
  Writeln('TO CHANGE: Push L (LIGHT) OR S (SOUND) OR B (BOTH)');
  Writeln('QUIT: Push "Q"');

  BitON(TopBeamLED);                          {Turn on Infared Beam}
  UpDateInfo;
  REPEAT
    Fault := FALSE;
    DelayTimeMs := 1000 + Random(2000);
    Writeln;
    writeln('Put your hand in the Infared Light Beam.');
    StartHrTimer(1);

    WHILE NOT Triggered(TopTrigger) DO
      BEGIN
        IF Keypressed THEN
          BEGIN
            Key := ReadKey;
            IF (Key = 'L') OR (Key = 'l')
              THEN
                BEGIN
                  UserLight := TRUE;
                  UserSound := FALSE;
                END;
            IF (Key = 'S') OR (Key = 's')
              THEN
                BEGIN
                  UserLight := FALSE;
                  UserSound := TRUE;
                END;
            IF (Key = 'B') OR (Key = 'b')
              THEN
                BEGIN
                  UserLight := TRUE;
                  UserSound := TRUE;
                END;
            IF (Key = 'Q') OR (Key = 'q') THEN
              BEGIN
                Window(1,1,80,25);
                ClrScr;
                Halt;
              END;
            UpdateInfo;
          END;{KeyPressed}
      END;{While NOT Triggered}

    StrobeHRTimers;
    writeln('--------------------------------');
    writeln('OK. NOW, WAIT FOR THE BEEP and LIGHT.');
    writeln('THEN TAKE YOUR HAND AWAY FAST!!');

    ReadHrTimer(1, Time1);

    Delay(DelayTimeMs);
    IF NOT Triggered(TopTrigger) THEN
      BEGIN
        Writeln;
        Writeln('!!!!!!!!!!! ( FAULT ) !!!!!!!!!!!');
        Writeln('YOU DIDN''T WAIT FOR THE SIGNAL!!!! NO FAIR!!!! ');
        Sound(800);
        Delay(400);
        Sound(200);
        Delay(500);
        NoSound;
        Fault := TRUE;
      END;

    IF NOT Fault THEN
      BEGIN
        IF UserSound THEN BitOn(ToneGen);
        IF UserLight THEN BitON(BlinkLED);
        Done := FALSE;
        StartHrTimer(2);
        REPEAT
           BEGIN
             Done := (NOT Triggered(TopTrigger));
           END
          UNTIL Done;
        StrobeHRTimers;
        BitOFF(ToneGen);
        BitOFF(BlinkLED);
        writeln(' **** YOU MOVED!! ****');
        ReadHrTimer(2, Time2);
        UpdateInfo;

        Str((Time1.us / 1000):8:2, Tbuff);
        Writeln('Time until START:  ms =', TBuff,
                '  Secs =' , Time1.ss:2 , ' Min =' , Time1.mm:2);
        Str((Time2.us / 1000000):8:4, Tbuff);
        SelectWindow(STATUS);
        Writeln('REACTION TIME:   ', Tbuff,
                '  Seconds');
        SelectWindow(MAIN);
    END;{ Not Fault}
  UNTIL KeyPressed;

  Window(1,1,80,25);
  ClrScr;

  {---------( END OF  PROGRAM )------------------------------------}


END.{Main Program}

